#!/bin/sh

tree=linux-2.6.7-bk20.src
prefix="source_ws"

dsttree="$prefix.$tree"

mkdir -p "$dsttree"
mkdir -p "$dsttree.patch"

echo "Note: $dsttree.patch/ directory must be empty, or you'll get duplicate patches"

{
# All files
#find "$tree"

# Files with bad whitespace
#grep -rl $'[ \t]$' "$tree"		# space or tab at EOL
#grep -rl  $'^[^ \t]       ' "$tree"	# char 7 spaces
#grep -rlF $' \t' "$tree"		# space tab
#grep -rlF $'\t ' "$tree"		# tab space 
#grep -rlF '        ' "$tree"		# 8 spaces

# Files with trailing whitespace in printk etc
# Why -i: DPRINTK and friends
grep -lri $'print.*[ \t]\\\\n' "$tree"
# '",' allows to filter out tons of false positives from asm()
grep -lr $'[ \t]\\\\n.*",' "$tree"

} \
| sort | uniq \
\
| while read -r file; do
    base=`basename "$file"`
    file2="$prefix.$file"

    # Names like irq.c.patch wouldn't be unique...
    patch="$base.patch"
    while test -e "$dsttree.patch/$patch"; do
	patch="$base-$RANDOM.patch"
    done

    echo "* $file"

    mkdir -p `dirname "$file2"`

    # Fix whitespace
    if false; then
    # begin 8spaces => begin tab
    # begin char 7spaces => begin char tab (repeat for 2,3,4...6 chars)
    # begin 1-7spaces tab => begin tab
    # tab 8spaces => tab tab (repeat)
    # tab 1-7spaces tab => tab tab (repeat)
    # trailing ws =>
    #| sed $'s/^        /\t/' \
    #| sed $'s/^\\([^ \t]\\)       /\\1\t/' \
    #| sed $'s/^ \\{1,7\\}\t/\t/' \
    #| sed $'s/\t        /\t\t/g' \
    #| sed $'s/\t \\{1,7\\}\t/\t\t/g' \
    #| sed $'s/[ \t]*$//' \
    cat "$file" \
    | sed $'s/^ \\{1,7\\}\t/\t/' \
    | sed -e $'s/\t        /\t\t/g' -e $'s/\t        /\t\t/g' \
          -e $'s/\t        /\t\t/g' -e $'s/\t        /\t\t/g' \
          -e $'s/\t        /\t\t/g' -e $'s/\t        /\t\t/g' \
    | sed -e $'s/\t \\{1,7\\}\t/\t\t/g' -e $'s/\t \\{1,7\\}\t/\t\t/g' \
          -e $'s/\t \\{1,7\\}\t/\t\t/g' -e $'s/\t \\{1,7\\}\t/\t\t/g' \
          -e $'s/\t \\{1,7\\}\t/\t\t/g' -e $'s/\t \\{1,7\\}\t/\t\t/g' \
    | sed $'s/[ \t]*$//' \
    > "$file2"
    fi

    # Fix whitespace in printks only
    # Multiple exprs handle stuff like:
    # printk(KERN_WARNING "ISILoad:Card%d rejected load header: \nAddress:0x%x \nCount:0x%x \nStatus:0x%x \n"
    # (multiple ' \n' occurences)
    # s/.../.../g would not help here
    cat "$file" \
    | sed -e $'s/[ \t]*\\\\n/\\\\n/' -e $'s/[ \t]*\\\\n/\\\\n/' \
          -e $'s/[ \t]*\\\\n/\\\\n/' -e $'s/[ \t]*\\\\n/\\\\n/' \
          -e $'s/[ \t]*\\\\n/\\\\n/' -e $'s/[ \t]*\\\\n/\\\\n/' \
          -e $'s/[ \t]*\\\\n/\\\\n/' -e $'s/[ \t]*\\\\n/\\\\n/' \
    > "$file2"

    if ! diff -up "$file" "$file2" >"$dsttree.patch/TEMP"; then
	mv "$dsttree.patch/TEMP" "$dsttree.patch/$patch"
    fi
done

rm "$dsttree.patch/TEMP" 2>/dev/null
echo "You may delete $dsttree now"
